import 'dart:io';

import 'package:bosun/bosun.dart';
import 'package:dcli/dcli.dart';
import 'package:me/functions/create/create_file.dart';
import 'package:me/functions/routes/add_page.dart';
import 'package:me/functions/routes/add_route.dart';
import 'package:me/samples/impl/binding.dart';
import 'package:me/samples/impl/provider.dart';
import 'package:recase/recase.dart';

import '../core/structure.dart';
import '../functions/binding/bindings.dart';
import '../functions/binding/controller.dart';
import '../functions/binding/find_file_by_type.dart';
import '../samples/impl/controller.dart';
import '../samples/impl/view.dart';
import '../utils/logger/log_utils.dart';
import '../utils/menu.dart';

class FeatureCommand extends Command {
  FeatureCommand()
      : super(
          command: 'feature',
          description: 'Create a new feature for Megaleios projects. 🔨',
          example: 'me feature login',
        );

  @override
  void run(List<String> args, Map<String, dynamic> flags) {
    if (args.isEmpty) {
      LogService.error("You must provide a feature name.");
      return;
    }
    _checkIfExist(featureName: args.first);
  }

  void _checkIfExist({required String featureName}) {
    final path = 'lib/app/modules/$featureName';
    if (Directory(path).existsSync()) {
      final menu = Menu(
        [
          'Yes',
          'No',
          'Insert new name',
        ],
        title:
            'The feature $featureName already exists, do you want to overwrite it?',
      );
      final result = menu.choose();
      if (result.index == 0) {
        _writeFiles(path, featureName, overwrite: true);
      } else if (result.index == 2) {
        var name = ask('What new name for the page?');
        _checkIfExist(featureName: name.trim().snakeCase);
      }
    } else {
      Directory(path).createSync(recursive: true);
      _writeFiles(path, featureName, overwrite: false);
    }
  }

  void _writeFiles(
    String path,
    String featureName, {
    required bool overwrite,
  }) {
    createFile(
      path: path,
      name: featureName,
      command: 'controller',
      folderName: 'controllers',
      sample: ControllerSample(
        '',
        featureName,
        overwrite: overwrite,
      ),
    );
    createFile(
      path: path,
      name: featureName,
      command: 'binding',
      folderName: 'bindings',
      sample: BindingSample(
        '',
        featureName,
        overwrite: overwrite,
      ),
    );
    createFile(
      path: path,
      name: featureName,
      command: 'view',
      folderName: 'views',
      sample: ViewSample(
        '',
        featureName,
        overwrite: overwrite,
      ),
    );
    final providerFile = createFile(
      path: 'lib/app/data',
      name: featureName,
      command: 'provider',
      folderName: 'providers',
      sample: ProviderSample(
        '',
        featureName,
        overwrite: overwrite,
      ),
    );
    Structure.pathToDirImport(providerFile.path);
    var binindingPath = findFileFromNameAndType(
      'lib/app/modules/$featureName/bindings/',
      featureName,
      typeFile: TypeFile.binding,
    );
    if (binindingPath == null) {
      LogService.error('No binding found for $featureName');
      return;
    }

    if (binindingPath.isNotEmpty) {
      addDependencyToBinding(binindingPath, featureName);
    }

    final controllerPath = findFileFromNameAndType(
      'lib/app/modules/$featureName/controllers/',
      featureName,
      typeFile: TypeFile.controller,
    );
    if (controllerPath == null) {
      LogService.error('No controller found for $featureName');
      return;
    }

    if (controllerPath.isNotEmpty) {
      addConstructorToController(
        path: controllerPath,
        providerName: featureName,
      );
    }

    if (!overwrite) {
      addRoute(featureName: featureName);
      addPage(featureName: featureName);
    }

    LogService.success("Created feature 🚀");
  }
}
